# Generated by Django 3.2.11 on 2022-07-22 19:12
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_egfr.calculators import EgfrCkdEpi, egfr_percent_change
from edc_visit_schedule.constants import DAY1
from tqdm import tqdm


def update_revised_egrf(apps, schema_editor):
    bloodresultrft_model_cls = apps.get_model("meta_subject.bloodresultsrft")
    subjectvisit_model_cls = apps.get_model("meta_subject.subjectvisit")
    subjectscreening_model_cls = apps.get_model("meta_screening.subjectscreening")
    total = bloodresultrft_model_cls.objects.all().count()
    for obj in tqdm(bloodresultrft_model_cls.objects.all(), total=total):
        subject_visit = subjectvisit_model_cls.objects.get(id=obj.subject_visit_id)
        subject_screening = subjectscreening_model_cls.objects.get(
            subject_identifier=subject_visit.subject_identifier
        )
        try:
            baseline_obj = bloodresultrft_model_cls.objects.get(
                subject_visit__subject_identifier=subject_visit.subject_identifier,
                subject_visit__visit_code=DAY1,
                subject_visit__visit_code_sequence=0,
            )
        except ObjectDoesNotExist:
            pass
        else:
            obj.old_egfr_value = obj.egfr_value
            obj.old_egfr_drop_value = obj.egfr_drop_value
            obj.egfr_value = EgfrCkdEpi(
                gender=subject_screening.gender,
                ethnicity=subject_screening.ethnicity,
                age_in_years=subject_screening.age_in_years,
                creatinine_value=obj.creatinine_value,
                creatinine_units=obj.creatinine_units,
            ).value
            obj.egfr_drop_value = egfr_percent_change(
                float(obj.egfr_value), float(baseline_obj.egfr_value)
            )
            obj.save_base(
                update_fields=[
                    "egfr_value",
                    "egfr_drop_value",
                    "old_egfr_value",
                    "old_egfr_drop_value",
                ]
            )


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0134_auto_20220722_2211"),
    ]

    operations = [migrations.RunPython(update_revised_egrf)]



# Generated by Django 3.2.13 on 2022-09-13 21:39
from django.db import migrations
from tqdm import tqdm


def update_rs_missing_ethnicity(apps, schema_editor):

    register_subject_model_cls = apps.get_model("edc_registration.registeredsubject")
    subject_screening_model_cls = apps.get_model("meta_screening.subjectscreening")
    subject_consent_model_cls = apps.get_model("meta_consent.subjectconsent")
    total = subject_consent_model_cls.objects.all().count()
    for subject_consent in tqdm(subject_consent_model_cls.objects.all(), total=total):
        subject_screening = subject_screening_model_cls.objects.get(
            subject_identifier=subject_consent.subject_identifier
        )
        register_subject = register_subject_model_cls.objects.get(
            subject_identifier=subject_consent.subject_identifier
        )
        register_subject.ethnicity = subject_screening.ethnicity
        register_subject.save(update_fields=["ethnicity"])


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0014_alter_subjectconsent_managers"),
        ("meta_subject", "0149_auto_20220913_2139"),
    ]

    operations = [migrations.RunPython(update_rs_missing_ethnicity)]



#!/usr/bin/env python
import logging
from datetime import datetime
from os.path import abspath, dirname, join
from zoneinfo import ZoneInfo

from edc_test_utils import DefaultTestSettings, func_main
from multisite import SiteID

from meta_edc.meta_version import PHASE_THREE

app_name = "meta_edc"
base_dir = dirname(abspath(__file__))

project_settings = DefaultTestSettings(
    calling_file=__file__,
    META_PHASE=PHASE_THREE,
    EDC_EGFR_DROP_NOTIFICATION_MODEL="meta_subject.egfrdropnotification",
    EDC_RANDOMIZATION_REGISTER_DEFAULT_RANDOMIZER=False,
    EDC_MNSI_ABNORMAL_FOOT_APPEARANCE_OBSERVATIONS_MODEL=(
        "meta_lists.abnormalfootappearanceobservations"
    ),
    EDC_QOL_EQ5D3L_MODEL="meta_subject.eq5d3l",
    ROOT_URLCONF="meta_edc.urls",
    EDC_AUTH_CODENAMES_WARN_ONLY=True,
    EDC_DX_REVIEW_LIST_MODEL_APP_LABEL="edc_dx_review",
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    SITE_ID=SiteID(default=10),
    SENTRY_ENABLED=False,
    INDEX_PAGE="localhost:8000",
    EXPORT_FOLDER=join(base_dir, "tests", "export"),
    SUBJECT_APP_LABEL="meta_subject",
    SUBJECT_SCREENING_MODEL="meta_screening.subjectscreening",
    SUBJECT_VISIT_MODEL="meta_subject.subjectvisit",
    SUBJECT_VISIT_MISSED_MODEL="meta_subject.subjectvisitmissed",
    SUBJECT_CONSENT_MODEL="meta_consent.subjectconsent",
    SUBJECT_REQUISITION_MODEL="meta_subject.subjectrequisition",
    EDC_BLOOD_RESULTS_MODEL_APP_LABEL="meta_subject",
    DEFENDER_ENABLED=False,
    DJANGO_LAB_DASHBOARD_REQUISITION_MODEL="meta_subject.subjectrequisition",
    ADVERSE_EVENT_ADMIN_SITE="meta_ae_admin",
    EDC_DX_LABELS=dict(hiv="HIV", dm="Diabetes", htn="Hypertension", chol="High Cholesterol"),
    ADVERSE_EVENT_APP_LABEL="meta_ae",
    EDC_NAVBAR_DEFAULT="meta_dashboard",
    EDC_PROTOCOL_STUDY_OPEN_DATETIME=datetime(2019, 4, 30, 0, 0, 0, tzinfo=ZoneInfo("UTC")),
    EDC_PROTOCOL_STUDY_CLOSE_DATETIME=datetime(
        2023, 12, 31, 23, 59, 59, tzinfo=ZoneInfo("UTC")
    ),
    DJANGO_LANGUAGES=dict(
        en="English",
        lg="Luganda",
        rny="Runyankore",
    ),
    DASHBOARD_BASE_TEMPLATES=dict(
        edc_base_template="edc_dashboard/base.html",
        listboard_base_template="meta_edc/base.html",
        dashboard_base_template="meta_edc/base.html",
        screening_listboard_template="meta_dashboard/screening/listboard.html",
        subject_listboard_template="meta_dashboard/subject/listboard.html",
        subject_dashboard_template="meta_dashboard/subject/dashboard.html",
        subject_review_listboard_template="edc_review_dashboard/subject_review_listboard.html",
    ),
    ETC_DIR=join(base_dir, "meta_edc", "tests", "etc"),
    EDC_BOOTSTRAP=3,
    EMAIL_BACKEND="django.core.mail.backends.locmem.EmailBackend",
    EMAIL_CONTACTS={
        "data_request": "someone@example.com",
        "data_manager": "someone@example.com",
        "tmg": "someone@example.com",
    },
    EMAIL_ENABLED=True,
    HOLIDAY_FILE=join(base_dir, "meta_edc", "tests", "holidays.csv"),
    LIVE_SYSTEM=False,
    EDC_RANDOMIZATION_LIST_PATH=join(base_dir, "meta_edc", "tests", "etc"),
    EDC_SITES_MODULE_NAME="meta_sites",
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    EDC_MNSI_MODEL="meta_subject.mnsi",
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        # "debug_toolbar",
        "django_extensions",
        "logentry_admin",
        "simple_history",
        "storages",
        "edc_action_item.apps.AppConfig",
        "edc_adverse_event.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_auth.apps.AppConfig",
        "edc_model_wrapper.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_data_manager.apps.AppConfig",
        "edc_consent.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_egfr.apps.AppConfig",
        "edc_export.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_fieldsets.apps.AppConfig",
        "edc_form_validators.apps.AppConfig",
        "edc_reportable.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_lab_dashboard.apps.AppConfig",
        "edc_label.apps.AppConfig",
        "edc_listboard.apps.AppConfig",
        "edc_locator.apps.AppConfig",
        "edc_reference.apps.AppConfig",
        "edc_pdf_reports.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_model_admin.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_pdutils.apps.AppConfig",
        "edc_pharmacy.apps.AppConfig",
        # "edc_pharmacy_dashboard.apps.AppConfig",
        "edc_protocol.apps.AppConfig",
        "edc_protocol_incident.apps.AppConfig",
        "edc_prn.apps.AppConfig",
        "edc_randomization.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_screening.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_list_data.apps.AppConfig",
        "edc_review_dashboard.apps.AppConfig",
        "edc_refusal.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_mnsi.apps.AppConfig",
        "edc_unblinding.apps.AppConfig",
        "edc_qol.apps.AppConfig",
        "edc_dx_review.apps.AppConfig",
        "edc_dx.apps.AppConfig",
        "canned_views.apps.AppConfig",
        "meta_auth.apps.AppConfig",
        "meta_consent.apps.AppConfig",
        "meta_lists.apps.AppConfig",
        "meta_dashboard.apps.AppConfig",
        "meta_labs.apps.AppConfig",
        "meta_rando.apps.AppConfig",
        "meta_reference.apps.AppConfig",
        "meta_subject.apps.AppConfig",
        "meta_visit_schedule.apps.AppConfig",
        "meta_ae.apps.AppConfig",
        "meta_prn.apps.AppConfig",
        "meta_export.apps.AppConfig",
        "meta_screening.apps.AppConfig",
        "meta_edc.apps.AppConfig",
    ],
    add_dashboard_middleware=True,
    add_lab_dashboard_middleware=True,
    add_adverse_event_dashboard_middleware=True,
).settings


def main():
    tests = [
        "tests",
        "meta_ae.tests",
        "meta_dashboard.tests",
        "meta_edc.tests",
        "meta_labs.tests",
        "meta_lists.tests",
        "meta_prn.tests",
        "meta_rando.tests",
        "meta_screening.tests",
        "meta_subject.tests",
        "meta_visit_schedule.tests",
    ]

    func_main(project_settings, *tests)


if __name__ == "__main__":
    logging.basicConfig()
    main()

# def main():
#     if not settings.configured:
#         settings.configure(**DEFAULT_SETTINGS)
#     django.setup()
#     tags = [t.split("=")[1] for t in sys.argv if t.startswith("--tag")]
#     failfast = True if [t for t in sys.argv if t == "--failfast"] else False
#     failures = DiscoverRunner(failfast=failfast, tags=tags).run_tests(
#         [
#             "tests",
#             "meta_ae.tests",
#             "meta_dashboard.tests",
#             "meta_edc.tests",
#             "meta_labs.tests",
#             "meta_lists.tests",
#             "meta_prn.tests",
#             "meta_rando.tests",
#             "meta_screening.tests",
#             "meta_subject.tests",
#             "meta_visit_schedule.tests",
#         ]
#     )
#     sys.exit(bool(failures))
#
#
# if __name__ == "__main__":
#     logging.basicConfig()
#     main()



# Generated by Django 3.2.11 on 2022-03-04 01:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0030_auto_20220128_1719"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt2_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt2_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> oral glucose solution was given",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                verbose_name="<u>Time</u> blood glucose measured 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt2_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-29 15:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0032_historicalegfrnotification_egfrnotification"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalegfrnotification",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalegfrnotification",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalegfrnotification",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalegfrnotification",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalegfrnotification",
            name="site",
        ),
        migrations.DeleteModel(
            name="EgfrNotification",
        ),
        migrations.DeleteModel(
            name="HistoricalEgfrNotification",
        ),
    ]



# Generated by Django 3.2.11 on 2022-07-19 21:55

from django.db import migrations
from django.db.models import F
from edc_reportable import PERCENT


def update_new_egfr_drop_field(apps, schema_editor):
    rft_model_cls = apps.get_model("meta_subject.bloodresultsrft")
    rft_model_cls.objects.all().update(
        egfr_drop_value=F("egfr_percent_change"), egfr_drop_units=PERCENT
    )
    rft_model_cls = apps.get_model("meta_subject.historicalbloodresultsrft")
    rft_model_cls.objects.all().update(
        egfr_drop_value=F("egfr_percent_change"), egfr_drop_units=PERCENT
    )


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0127_auto_20220720_0053"),
    ]

    operations = [migrations.RunPython(update_new_egfr_drop_field)]



from decimal import Decimal

from django.contrib import admin, messages
from django.contrib.messages import ERROR, SUCCESS
from django.core.exceptions import ObjectDoesNotExist
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_offstudy.exceptions import OffstudyError
from edc_pharmacy.exceptions import NextStudyMedicationError, RefillCreatorError
from edc_pharmacy.model_mixins.study_medication_crf_model_mixin import (
    StudyMedicationError,
)
from edc_pharmacy.models import DosageGuideline, Formulation, Medication
from edc_visit_schedule.utils import is_baseline

from meta_pharmacy.constants import METFORMIN

from ..admin_site import meta_subject_admin
from ..forms import StudyMedicationForm
from ..models import StudyMedication
from .modeladmin import CrfModelAdmin


@admin.register(StudyMedication, site=meta_subject_admin)
class StudyMedicationAdmin(CrfModelAdmin):

    actions = ["create_or_update_rx_refills"]

    form = StudyMedicationForm

    autocomplete_fields = [
        "dosage_guideline",
        "formulation",
    ]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "refill",
                )
            },
        ),
        (
            "Details",
            {
                "fields": (
                    "refill_start_datetime",
                    "dosage_guideline",
                    "formulation",
                    "refill_to_next_visit",
                    "refill_end_datetime",
                    "special_instructions",
                ),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "formulation": admin.VERTICAL,
        "refill": admin.VERTICAL,
        "refill_to_next_visit": admin.VERTICAL,
    }

    @admin.action(permissions=["view"], description="Create or update RX Refills")
    def create_or_update_rx_refills(self, request, queryset):
        updated = 0
        errors = 0
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.get(medication=medication, strength=500)
        dosage_guideline_baseline = DosageGuideline.objects.get(
            medication=medication, dose=Decimal("1000.0")
        )
        dosage_guideline_followup = DosageGuideline.objects.get(
            medication=medication, dose=Decimal("2000.0")
        )
        for obj in queryset:
            obj.refill_identifier = obj.id
            if not obj.formulation:
                obj.formulation = formulation
            if not obj.dosage_guideline:
                if is_baseline(obj.related_visit):
                    obj.dosage_guideline = dosage_guideline_baseline
                else:
                    obj.dosage_guideline = dosage_guideline_followup
            updated, errors = self.rx_refills_save_or_raise(request, obj, updated, errors)
        messages.add_message(
            request, SUCCESS, f"Updated {updated} documents with {errors} errors."
        )

    @staticmethod
    def rx_refills_save_or_raise(request, obj, updated, errors):
        try:
            obj.save()
        except NextStudyMedicationError as e:
            messages.add_message(request, ERROR, f"NextStudyMedicationError: {e}")
            errors += 1
        except ObjectDoesNotExist as e:
            messages.add_message(
                request,
                ERROR,
                (
                    f"ObjectDoesNotExist: {obj.subject_identifier}, "
                    f"{obj.related_visit}, {str(e)}"
                ),
            )
            errors += 1
        except StudyMedicationError as e:
            messages.add_message(request, ERROR, f"StudyMedicationError: {e}")
            errors += 1
        except RefillCreatorError as e:
            messages.add_message(request, ERROR, f"RefillCreatorError: {e}")
            errors += 1
        except OffstudyError as e:
            messages.add_message(request, ERROR, f"OffstudyError: {e}")
            errors += 1
        else:
            updated += 1
        return updated, errors



# Generated by Django 3.2.13 on 2022-08-26 01:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0046_auto_20220826_0322"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="endofstudy",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicallosstofollowup",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicaloffschedule",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicaloffschedulepostnatal",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicaloffschedulepregnancy",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicaloffstudymedication",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalpregnancynotification",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalprotocolincident",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalsubjecttransfer",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="losstofollowup",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="offschedule",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="offschedulepostnatal",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="offschedulepregnancy",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="offstudymedication",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="pregnancynotification",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="protocolincident",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="subjecttransfer",
            name="tracking_identifier",
        ),
    ]



from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import fbc_panel
from edc_lab_results import BLOOD_RESULTS_FBC_ACTION
from edc_lab_results.model_mixins import (
    BloodResultsModelMixin,
    HaemoglobinModelMixin,
    HctModelMixin,
    MchcModelMixin,
    MchModelMixin,
    McvModelMixin,
    PlateletsModelMixin,
    RbcModelMixin,
    WbcModelMixin,
)
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsFbc(
    CrfWithActionModelMixin,
    CrfWithRequisitionModelMixin,
    HaemoglobinModelMixin,
    HctModelMixin,
    RbcModelMixin,
    WbcModelMixin,
    PlateletsModelMixin,
    MchModelMixin,
    MchcModelMixin,
    McvModelMixin,
    BloodResultsModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_FBC_ACTION
    tracking_identifier_prefix = "FB"

    lab_panel = fbc_panel

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: FBC"
        verbose_name_plural = "Blood Results: FBC"



# Generated by Django 2.2.7 on 2020-01-20 13:22

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0024_auto_20191024_1000"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("meta_subject", "0013_auto_20200119_0013"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalBloodResultsLipid",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("tracking_identifier", models.CharField(db_index=True, max_length=30)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "results_abnormal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Are any of the above results abnormal?",
                    ),
                ),
                (
                    "results_reportable",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        help_text="If YES, this value will open Adverse Event Form.<br/><br/>Note: On Day 1 only abnormal bloods should not be reported as adverseevents.",
                        max_length=25,
                        verbose_name="If any results are abnormal, are results within grade III or above?",
                    ),
                ),
                ("summary", models.TextField(blank=True, null=True)),
                (
                    "lipid_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="Result Report Date and Time",
                    ),
                ),
                (
                    "ldl",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="LDL",
                    ),
                ),
                (
                    "ldl_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "ldl_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "ldl_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                (
                    "hdl",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="HDL",
                    ),
                ),
                (
                    "hdl_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "hdl_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "hdl_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                (
                    "trig",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="Triglycerides",
                    ),
                ),
                (
                    "trig_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "trig_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "trig_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "lipid_requisition",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        help_text="Start typing the requisition identifier or select one from this visit",
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.SubjectRequisition",
                        verbose_name="Requisition",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.SubjectVisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Blood Result: LIPID",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="BloodResultsLipid",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("tracking_identifier", models.CharField(max_length=30, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "results_abnormal",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Are any of the above results abnormal?",
                    ),
                ),
                (
                    "results_reportable",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        help_text="If YES, this value will open Adverse Event Form.<br/><br/>Note: On Day 1 only abnormal bloods should not be reported as adverseevents.",
                        max_length=25,
                        verbose_name="If any results are abnormal, are results within grade III or above?",
                    ),
                ),
                ("summary", models.TextField(blank=True, null=True)),
                (
                    "lipid_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="Result Report Date and Time",
                    ),
                ),
                (
                    "ldl",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="LDL",
                    ),
                ),
                (
                    "ldl_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "ldl_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "ldl_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                (
                    "hdl",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="HDL",
                    ),
                ),
                (
                    "hdl_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "hdl_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "hdl_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                (
                    "trig",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        max_digits=8,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999),
                        ],
                        verbose_name="Triglycerides",
                    ),
                ),
                (
                    "trig_units",
                    models.CharField(
                        blank=True,
                        choices=[("mmol/L", "mmol/L")],
                        max_length=15,
                        null=True,
                        verbose_name="units",
                    ),
                ),
                (
                    "trig_abnormal",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        null=True,
                        verbose_name="abnormal",
                    ),
                ),
                (
                    "trig_reportable",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("N/A", "Not applicable"),
                            ("3", "Yes, grade 3"),
                            ("4", "Yes, grade 4"),
                            ("No", "Not reportable"),
                            ("Already reported", "Already reported"),
                            ("present_at_baseline", "Present at baseline"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="reportable",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "lipid_requisition",
                    models.ForeignKey(
                        blank=True,
                        help_text="Start typing the requisition identifier or select one from this visit",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="lipid",
                        to="meta_subject.SubjectRequisition",
                        verbose_name="Requisition",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.ActionItem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.SubjectVisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blood Result: LIPID",
                "verbose_name_plural": "Blood Results: LIPID",
                "abstract": False,
            },
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AddIndex(
            model_name="bloodresultslipid",
            index=models.Index(
                fields=["subject_visit", "site", "id"],
                name="meta_subjec_subject_c41a0b_idx",
            ),
        ),
    ]



from django import forms
from django.utils.safestring import mark_safe
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab_panel.panels import rft_panel
from edc_lab_results.form_validator_mixins import BloodResultsFormValidatorMixin
from edc_reportable import BmiFormValidatorMixin

from ...models import BloodResultsRft


class BloodResultsRftFormValidator(
    BloodResultsFormValidatorMixin, BmiFormValidatorMixin, CrfFormValidator
):
    panel = rft_panel

    def clean(self):
        super().clean()
        self.validate_bmi()


class BloodResultsRftForm(ActionItemCrfFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = BloodResultsRftFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = BloodResultsRft
        fields = "__all__"
        help_texts = {
            "action_identifier": "(read-only)",
            "egfr_value": mark_safe(  # nosec B308
                "Calculated using 2009 CKD-EPI Creatinine. "
                "See https://nephron.com/epi_equation"
            ),
            "egfr_drop_value": mark_safe(  # nosec B308
                "Calculated using 2009 CKD-EPI Creatinine. "
                "See https://nephron.com/epi_equation"
            ),
        }



from copy import deepcopy
from decimal import Decimal

from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_constants.constants import FEMALE, NO, NOT_APPLICABLE, POS, TBD, YES
from edc_reportable import (
    MICROMOLES_PER_LITER,
    MILLIMOLES_PER_LITER,
    ConversionNotHandled,
)
from edc_utils.date import get_utcnow

from meta_screening.constants import (
    EGFR_LT_45,
    EGFR_NOT_CALCULATED,
    FBG_OGTT,
    FBG_OGTT_INCOMPLETE,
    NORMAL_FBG_OGTT,
)
from meta_screening.models import ScreeningPartOne, ScreeningPartThree, ScreeningPartTwo

from ..options import (
    get_part_one_eligible_options,
    get_part_three_eligible_options,
    get_part_two_eligible_options,
)


class TestScreeningPartThree(TestCase):
    def setUp(self):
        """Complete parts one and two first ..."""
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        obj = ScreeningPartOne(**part_one_eligible_options)
        obj.save()
        obj.refresh_from_db()
        self.screening_identifier = obj.screening_identifier

        obj = ScreeningPartTwo.objects.get(screening_identifier=self.screening_identifier)
        for k, v in part_two_eligible_options.items():
            getattr(obj, k)
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()

        # assert eligible for part one criteria
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertIsNone(obj.reasons_ineligible_part_one)

        # assert eligible for part two criteria
        self.assertEqual(obj.eligible_part_two, YES)
        self.assertIsNone(obj.reasons_ineligible_part_two)

        # assert eligiblility to be determined for part three
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertIsNotNone(obj.reasons_ineligible_part_three)

        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

    def get_screening_part_three_obj(self):
        """Returns an SubjectScreening obj.

        Remember this is just a proxy model for SubjectScreening.
        """
        return ScreeningPartThree.objects.get(
            screening_identifier=self.screening_identifier,
        )

    def test_eligible_part_three_defaults_phase_three(self):
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        part_three_eligible_options["fbg_value"] = 6.9
        part_three_eligible_options["ogtt_value"] = 7.8
        self._test_eligible(part_three_eligible_options)

    def _test_eligible(self, part_three_eligible_options):
        obj = self.get_screening_part_three_obj()
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        self.assertIsNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)

    def test_eligible_datetime_does_not_change_on_resave(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()
        eligibility_datetime = obj.eligibility_datetime
        obj.save()
        obj.refresh_from_db()
        self.assertEqual(eligibility_datetime, obj.eligibility_datetime)

    def test_eligible2_phase_three(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)

        self.assertIsNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)
        obj.severe_htn = YES
        obj.save()
        obj.refresh_from_db()
        self.assertIn("Severe HTN", obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, NO)
        obj.severe_htn = NO
        obj.save()
        obj.refresh_from_db()
        self.assertIsNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)

    def test_eligible2_phase_three_missing_ogtt(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.ogtt_base_datetime = None
        obj.ogtt_datetime = None
        obj.ogtt_units = None
        obj.ogtt_value = None
        obj.save()
        obj.refresh_from_db()
        self._test_eligible2(obj, FBG_OGTT_INCOMPLETE, FBG_OGTT)

    def test_eligible2_phase_three_ogtt2_not_performed_is_ok(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.eligible_part_three, YES)

    def test_eligible2_phase_three_repeat_ogtt2_updates_converted(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()

        obj.repeat_glucose_opinion = YES
        obj.ogtt2_base_datetime = obj.ogtt_base_datetime + relativedelta(days=3)
        obj.ogtt2_datetime = obj.ogtt_datetime + relativedelta(days=3)
        obj.ogtt2_units = obj.ogtt_units
        obj.ogtt2_value = 6.9
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)

    def test_eligible2_phase_three_by_repeat_ogtt(self):
        obj = self.get_screening_part_three_obj()
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())

        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj.refresh_from_db()

        # eligible by single OGTT
        self.assertIsNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)

        # repeat OGTT, result in range
        # cannot just repeat OGTT2
        obj.repeat_glucose_opinion = YES
        obj.ogtt2_base_datetime = obj.ogtt_base_datetime + relativedelta(days=3)
        obj.ogtt2_datetime = obj.ogtt_datetime + relativedelta(days=3)
        obj.ogtt2_units = obj.ogtt_units
        obj.ogtt2_value = Decimal("8.1000")
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)
        self.assertIsNotNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, TBD)

        # also repeat the FBG
        obj.fbg2_datetime = obj.ogtt_datetime + relativedelta(days=3)
        obj.fbg2_units = obj.fbg_units
        obj.fbg2_value = Decimal("6.1000")
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)
        self.assertIsNone(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)

        # repeat OGTT, result too low
        obj.ogtt2_value = Decimal("5.5000")
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)
        # still eligible because the FBG qualifies
        self.assertEqual(obj.eligible_part_three, YES)
        self.assertIsNone(obj.reasons_ineligible_part_three)

        # repeat FBG, result too low
        obj.fbg2_value = Decimal("5.5000")
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_fbg_value, obj.fbg_value)
        self.assertEqual(obj.converted_fbg2_value, obj.fbg2_value)
        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)
        # not eligible because the FBG2 and OGTT2 do not qualify
        self.assertEqual(obj.eligible_part_three, NO)
        self.assertIsNotNone(obj.reasons_ineligible_part_three)
        self.assertIn(NORMAL_FBG_OGTT, obj.reasons_ineligible_part_three)

        obj.ogtt2_value = Decimal("7.9000")
        obj.save()
        obj.refresh_from_db()

        self.assertEqual(obj.converted_fbg_value, obj.fbg_value)
        self.assertEqual(obj.converted_fbg2_value, obj.fbg2_value)
        self.assertEqual(obj.converted_ogtt_value, obj.ogtt_value)
        self.assertEqual(obj.converted_ogtt2_value, obj.ogtt2_value)
        self.assertEqual(obj.eligible_part_three, YES)
        self.assertIsNone(obj.reasons_ineligible_part_three)

    def _test_eligible2(self, obj, incomplete_reason: str, ineligible_reason: str):
        self.assertIsNone(obj.reasons_ineligible_part_one)
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertIsNone(obj.reasons_ineligible_part_two)
        self.assertEqual(obj.eligible_part_two, YES)

        self.assertEqual(obj.reasons_ineligible_part_three, incomplete_reason)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.hba1c_performed = YES
        obj.hba1c_value = 7.0
        obj.creatinine_performed = NO
        obj.save()

        self.assertEqual(obj.reasons_ineligible_part_three, incomplete_reason)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.creatinine_performed = YES
        obj.creatinine_value = 50
        obj.creatinine_units = MICROMOLES_PER_LITER
        obj.save()

        self.assertEqual(obj.reasons_ineligible_part_three, incomplete_reason)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.fasting = NO
        obj.save()

        self.assertEqual(obj.reasons_ineligible_part_three, incomplete_reason)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.fasting = YES
        obj.fasting_duration_str = "8h"
        obj.fbg_value = 6.9
        obj.fbg_units = MILLIMOLES_PER_LITER
        obj.fbg_datetime = get_utcnow()
        obj.save()

        self.assertEqual(obj.reasons_ineligible_part_three, incomplete_reason)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.ogtt_base_datetime = get_utcnow()
        obj.ogtt_value = 3.0
        obj.ogtt_units = MICROMOLES_PER_LITER
        obj.ogtt_datetime = get_utcnow()
        try:
            obj.save()
        except ConversionNotHandled:
            pass
        else:
            self.fail("ConversionNotHandled unexpectedly not raised.")

        obj.refresh_from_db()
        obj.ogtt_units = MILLIMOLES_PER_LITER
        obj.save()

        self.assertIn(ineligible_reason, obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.ogtt_base_datetime = get_utcnow()
        obj.ogtt_value = 7.8
        obj.ogtt_units = MILLIMOLES_PER_LITER
        obj.ogtt_datetime = get_utcnow()
        obj.save()

        self.assertFalse(obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, YES)
        self.assertTrue(obj.eligible)
        self.assertFalse(obj.consented)

    def test_yes_eligible_egfr_not_calculated_phase_three(self):
        """Is eligible, since phase three will allow for EGFR to be considered
        late exclusion.
        """
        fbg_value = 6.9
        ogtt_value = 7.8
        obj = self._test_egfr_not_calculated(fbg_value, ogtt_value)
        self.assertNotIn(EGFR_NOT_CALCULATED, obj.reasons_ineligible_part_three or "")
        self.assertEqual(obj.eligible_part_three, YES)
        self.assertTrue(obj.eligible)
        self.assertFalse(obj.consented)

    def _test_egfr_not_calculated(self, fbg_value, ogtt_value):
        obj = self.get_screening_part_three_obj()
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertFalse(obj.reasons_ineligible_part_one)
        self.assertEqual(obj.eligible_part_two, YES)
        self.assertFalse(obj.reasons_ineligible_part_two)

        obj.fbg_value = fbg_value
        obj.ogtt_value = ogtt_value

        obj.part_three_report_datetime = get_utcnow()
        obj.weight = 65
        obj.height = 110
        obj.hba1c_performed = YES
        obj.hba1c_value = 7.0
        obj.creatinine_performed = NO
        obj.fasting = YES
        obj.fasting_duration_str = "8h"
        obj.fbg_units = MILLIMOLES_PER_LITER
        obj.fbg_datetime = get_utcnow()
        obj.ogtt_base_datetime = get_utcnow()
        obj.ogtt_units = MILLIMOLES_PER_LITER
        obj.ogtt_datetime = get_utcnow()
        obj.severe_htn = NO
        obj.save()
        return obj

    def test_not_eligible_egfr_less_than_45_phase_three(self):
        fbg_value = 6.9
        ogtt_value = 7.8
        self._test_not_eligible_egfr_less_than_45(fbg_value, ogtt_value)

    def _test_not_eligible_egfr_less_than_45(self, fbg_value, ogtt_value):

        obj = self.get_screening_part_three_obj()
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertFalse(obj.reasons_ineligible_part_one)
        self.assertEqual(obj.eligible_part_two, YES)
        self.assertFalse(obj.reasons_ineligible_part_two)

        obj.fbg_value = fbg_value
        obj.ogtt_value = ogtt_value

        # defaults
        obj.creatinine_performed = NO
        obj.creatinine_units = MICROMOLES_PER_LITER
        obj.creatinine_value = 200
        obj.fasting = YES
        obj.fasting_duration_str = "8h"
        obj.hba1c_performed = YES
        obj.hba1c_value = 7.0
        obj.height = 110
        obj.fbg_datetime = get_utcnow()
        obj.fbg_units = MILLIMOLES_PER_LITER
        obj.ogtt_base_datetime = get_utcnow()
        obj.ogtt_datetime = get_utcnow()
        obj.ogtt_units = MILLIMOLES_PER_LITER
        obj.part_three_report_datetime = get_utcnow()
        obj.severe_htn = NO
        obj.weight = 65
        obj.save()

        self.assertIn(EGFR_LT_45, obj.reasons_ineligible_part_three)
        self.assertEqual(obj.eligible_part_three, NO)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

    def test_pregnancy_detected_in_p3(self):
        """P1 pregnant==NO, p3"""
        # pregnant==NO
        # urine_bhcg_performed == YES
        # urine_bhcg_value == POS

        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_one_eligible_options.update(gender=FEMALE, pregnant=NO)
        model_obj = ScreeningPartOne(**part_one_eligible_options)
        model_obj.save()
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())

        for k, v in part_two_eligible_options.items():
            setattr(model_obj, k, v)
        model_obj.save()
        self.assertEqual(YES, model_obj.eligible_part_one)
        self.assertEqual(YES, model_obj.eligible_part_two)
        self.assertEqual(TBD, model_obj.eligible_part_three)

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        model_obj = self.get_screening_part_three_obj()
        for k, v in part_three_eligible_options.items():
            setattr(model_obj, k, v)

        model_obj.urine_bhcg_performed = NO
        model_obj.urine_bhcg_value = NOT_APPLICABLE
        model_obj.urine_bhcg_date = None
        model_obj.save()
        self.assertIsNone(model_obj.reasons_ineligible_part_three)
        self.assertEqual(model_obj.eligible_part_three, YES)

        model_obj.urine_bhcg_performed = YES
        model_obj.urine_bhcg_value = POS
        model_obj.urine_bhcg_date = model_obj.part_three_report_datetime
        model_obj.save()
        self.assertIn("UPT positive", model_obj.reasons_ineligible_part_three)
        self.assertEqual(model_obj.eligible_part_three, NO)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_he import education_fieldset, education_radio_fields

from ..admin_site import meta_subject_admin
from ..forms import HealthEconomicsSimpleForm
from ..models import HealthEconomicsSimple
from .modeladmin import CrfModelAdmin


@admin.register(HealthEconomicsSimple, site=meta_subject_admin)
class HealthEconomicsSimpleAdmin(CrfModelAdmin):

    form = HealthEconomicsSimpleForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "marital_status",
                    "occupation",
                )
            },
        ),
        education_fieldset,
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {**education_radio_fields}



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_glucose.form_validators import FbgOgttFormValidatorMixin

from ..models import Glucose


class GlucoseFormValidator(FbgOgttFormValidatorMixin, CrfFormValidator):
    def clean(self):
        self.validate_glucose_testing_matrix()


class GlucoseForm(CrfModelFormMixin, ActionItemCrfFormMixin, forms.ModelForm):
    form_validator_cls = GlucoseFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = Glucose
        fields = "__all__"



# Generated by Django 2.2.7 on 2019-11-19 20:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0008_auto_20191115_0132"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsglu",
            name="glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="glucose_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
            ),
        ),
    ]



from django.db import models
from django.db.models import PROTECT
from django.utils.safestring import mark_safe
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from meta_lists.models import ArvRegimens, Symptoms

from ..choices import GRADE34_CHOICES
from ..constants import FOLLOWUP_EXAMINATION_ACTION
from ..model_mixins import CrfWithActionModelMixin


class FollowupExamination(CrfWithActionModelMixin, BaseUuidModel):

    action_name = FOLLOWUP_EXAMINATION_ACTION

    tracking_identifier_prefix = "FU"

    # 4a
    symptoms = models.ManyToManyField(
        Symptoms,
        related_name="symptoms",
        verbose_name=(
            "Since the participant's last appointment have they experienced "
            "any of the following symptoms"
        ),
        help_text="either at this hospital or at a different clinic",
    )

    # 4b
    symptoms_sought_care = models.ManyToManyField(
        Symptoms,
        related_name="symptoms_sought_care",
        verbose_name="Did they seek care from any health worker for these symptoms",
    )

    # 4a
    symptoms_g3 = models.ManyToManyField(
        Symptoms,
        related_name="symptoms_g3",
        verbose_name="For these symptoms, were any grade 3 events",
        help_text=(
            "Refer to DAIDS toxicity table. " "Please complete Serious Adverse Event form"
        ),
    )
    symptoms_g3_detail = models.TextField(
        verbose_name="Please provide details on any of the Grade 3 symptoms above.",
        null=True,
        blank=True,
    )

    # 4a
    symptoms_g4 = models.ManyToManyField(
        Symptoms,
        related_name="symptoms_g4",
        verbose_name="For these symptoms, were any grade 4 events",
        help_text=(
            "Refer to DAIDS toxicity table. " "Please complete Serious Adverse Event form"
        ),
    )
    symptoms_g4_detail = models.TextField(
        verbose_name="Please provide details on any of the Grade 4 symptoms above.",
        null=True,
        blank=True,
    )

    # 4c
    symptoms_detail = models.TextField(
        verbose_name="Please provide details on any of the symptoms above.",
        null=True,
        blank=True,
    )

    # 5a
    attended_clinic = models.CharField(
        verbose_name=(
            "Since the participant's last visit did they attend any other clinic or hospital "
            "for care for any reason"
        ),
        max_length=25,
        choices=YES_NO,
        help_text="Includes other routine appointments, e.g. BP check or family planning",
    )

    # 5b
    admitted_hospital = models.CharField(
        verbose_name="If YES, were they admitted to hospital?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    # 5c
    attended_clinic_detail = models.TextField(
        verbose_name=(
            "If YES, attend other clinic or hospital, " "please provide details of this event"
        ),
        help_text=(
            "If the participant was given a referral letter or "
            "discharge summary record details here"
        ),
        null=True,
        blank=True,
    )

    attended_clinic_sae = models.CharField(
        verbose_name=mark_safe(  # nosec B308
            "Does the event constitute a <u>Serious Adverse Event</u>"
        ),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="If YES, submit a <u>Serious Adverse Event</u> Form",
    )

    # 5d
    prescribed_medication = models.CharField(
        verbose_name=(
            "Was the participant prescribed any other medication at "
            "this clinic or hospital visit?"
        ),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    # 5d
    prescribed_medication_detail = models.TextField(
        verbose_name=(
            "If YES, prescribed any other medication, " "please provide details of this visit"
        ),
        null=True,
        blank=True,
    )

    # TODO: Add diagnoses m2m with other inseatd of any_other_problems

    # 6a
    any_other_problems = models.CharField(
        verbose_name=(
            "Since your last visit has the participant experienced any other "
            "medical or health problems NOT listed above"
        ),
        max_length=25,
        choices=YES_NO,
    )

    # 6a
    any_other_problems_detail = models.TextField(
        verbose_name="If YES, please provide details of the event",
        null=True,
        blank=True,
    )

    any_other_problems_sae = models.CharField(
        verbose_name="Does this event constitute an Adverse Event?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    any_other_problems_sae_grade = models.CharField(
        verbose_name="If YES, what grade?",
        choices=GRADE34_CHOICES,
        max_length=25,
        default=NOT_APPLICABLE,
        help_text="If YES, grade 3 or 4, submit Serious Adverse Event form",
    )

    # 7a
    art_change = models.CharField(
        verbose_name=(
            "Since the participant's last visit has there "
            "been any change in their HIV medication?"
        ),
        max_length=25,
        choices=YES_NO,
    )

    # 7b
    art_change_reason = models.TextField(
        verbose_name="If YES, please provide reason for change", null=True, blank=True
    )

    # 7c
    art_new_regimen = models.ForeignKey(
        ArvRegimens,
        on_delete=PROTECT,
        verbose_name="Please indicate new regimen",
        related_name="art_new_regimen",
        null=True,
        blank=True,
    )

    art_new_regimen_other = OtherCharField()

    abdominal_tenderness = models.CharField(
        verbose_name="Abdominal tenderness", max_length=25, choices=YES_NO
    )

    enlarged_liver = models.CharField(
        verbose_name="Enlarged liver", max_length=25, choices=YES_NO
    )

    jaundice = models.CharField(verbose_name="Jaundice", max_length=25, choices=YES_NO)

    comment = models.TextField(
        verbose_name=(
            "Comment on the clinical course, any other symptoms present, "
            "assessment and management plan"
        )
    )

    lactic_acidosis = models.CharField(
        verbose_name="Do you think the participant has lactic acidosis?",
        max_length=25,
        choices=YES_NO,
        help_text=(
            "Submit SAE form. Classical signs of lactic acidosis include: abdominal "
            "or stomach discomfort, decreased appetite, diarrhoea, fast or shallow "
            "breathing, a general feeling of discomfort, muscle pain or cramping; and "
            "unusual sleepiness, fatigue, or weakness."
        ),
    )

    hepatomegaly = models.CharField(
        verbose_name="Do you think the participant has hepatomegaly with steatosis?",
        max_length=25,
        choices=YES_NO,
        help_text=(
            "Submit SAE form. This condition often does not have any clinical "
            "signs and symptoms, it may present with an enlarge liver on examination, "
            "and symptoms of fatigue and right upper abdominal pain. The risk of "
            "developing this condition is higher in patients who are obese and who have "
            "type 2 diabetes or metabolic syndrome"
        ),
    )

    referral = models.CharField(
        verbose_name="Is the participant being referred", max_length=25, choices=YES_NO
    )

    referral_reason = models.TextField(
        verbose_name="If YES, where are they being referred to",
        null=True,
        blank=True,
        help_text=(
            "Note: remind participant that admission or discharge information "
            "will be needed at the next follow up visit."
        ),
    )

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Clinic follow up: Examination"
        verbose_name_plural = "Clinic follow up: Examination"



# Generated by Django 3.2.13 on 2022-09-13 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0148_auto_20220908_1826"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="order_or_update_next",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Order, or update, refill for next scheduled visit?",
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="order_or_update_next",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Order, or update, refill for next scheduled visit?",
            ),
        ),
    ]



from django import forms
from edc_constants.constants import NO, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import MalariaTest


class MalariaTestFormValidator(CrfFormValidator):
    def clean(self):

        self.applicable_if(YES, field="performed", field_applicable="diagnostic_type")

        self.required_if(NO, field="performed", field_required="not_performed_reason")

        self.applicable_if(YES, field="performed", field_applicable="result")


class MalariaTestForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = MalariaTestFormValidator

    class Meta:
        model = MalariaTest
        fields = "__all__"



from edc_egfr.constants import EGFR_DROP_NOTIFICATION_ACTION
from edc_egfr.model_mixins import EgfrDropNotificationModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfWithActionModelMixin


class EgfrDropNotification(
    EgfrDropNotificationModelMixin,
    CrfWithActionModelMixin,
    BaseUuidModel,
):

    action_name = EGFR_DROP_NOTIFICATION_ACTION

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "eGFR Drop Notification"
        verbose_name_plural = "eGFR Drop Notifications"



# Generated by Django 3.2.6 on 2021-09-10 16:11

from django.db import migrations
from django.db.migrations import RunPython


def convert_violation_type_to_fk(apps, schema_editor):
    model_cls = apps.get_model("meta_prn", "protocoldeviationviolation")
    list_model_cls = apps.get_model("edc_protocol_incident", "protocolviolations")
    for obj in model_cls.objects.all():
        violation_type = list_model_cls.objects.get(name=obj.violation_type)
        obj.violation = violation_type
        obj.violation_other = obj.violation_type_other
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0011_auto_20210910_1911"),
    ]

    operations = [RunPython(convert_violation_type_to_fk)]



from django.db import models
from edc_constants.choices import NORMAL_ABNORMAL, PRESENT_ABSENT, YES_NO
from edc_data_manager.modeladmin_mixins import DataManagerModelAdminMixin
from edc_model import models as edc_models

from ..choices import FUNDOSCOPY_CHOICES
from ..model_mixins import CrfModelMixin


class Complications(DataManagerModelAdminMixin, CrfModelMixin, edc_models.BaseUuidModel):

    cataracts = models.CharField(
        verbose_name="Presence of cataracts", max_length=15, choices=YES_NO
    )

    fundoscopy = models.CharField(
        verbose_name="Fundoscopy", max_length=35, choices=FUNDOSCOPY_CHOICES
    )

    achilles_tendon_reflex = models.CharField(
        verbose_name="Achilles tendon reflex", max_length=15, choices=PRESENT_ABSENT
    )

    foot_pin_prick = models.CharField(
        verbose_name="Pin prick testing on foot",
        max_length=15,
        choices=NORMAL_ABNORMAL,
        help_text="Can the patient distinguish between sharp and non-sharp?",
    )

    foot_light_touch = models.CharField(
        verbose_name="Light touch",
        max_length=15,
        choices=NORMAL_ABNORMAL,
        help_text="Can the patient feel light pressure on the dorsum of the foot?",
    )

    temperature_perception = models.CharField(
        verbose_name="Temperature perception",
        max_length=15,
        choices=NORMAL_ABNORMAL,
        help_text=(
            "Can the patient distinguish between temperature on the dorsum of the foot?"
        ),
    )

    dorsalis_pedis_pulse = models.CharField(
        verbose_name="Dorsalis pedis pulse", max_length=15, choices=PRESENT_ABSENT
    )

    posterior_tibial_pulse = models.CharField(
        verbose_name="Posterior tibial pulse", max_length=15, choices=PRESENT_ABSENT
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Presence of Complications"
        verbose_name_plural = "Presence of Complications"



from edc_subject_dashboard.views import SubjectDashboardView


class DashboardView(SubjectDashboardView):

    consent_model = "meta_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "meta_subject.subjectvisit"
    history_button_label = "Audit"



# Generated by Django 4.0.7 on 2022-08-23 13:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0139_alter_bloodresultsins_ins_units_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="urinepregnancy",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
    ]



from copy import deepcopy

from django.test import TestCase
from edc_constants.constants import COMPLETE, NEVER, NO, YES
from model_bakery.baker import make_recipe

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.forms import MedicationAdherenceForm


class TestMedicationAdherence(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()

        self.subject_visit = self.get_subject_visit()

        self.data = dict(
            site=None,
            crf_status=COMPLETE,
            subject_visit=None,
            report_datetime=None,
            visual_score_slider=90,
            visual_score_confirmed=90,
            last_missed_pill=NEVER,
            pill_count_performed=YES,
            pill_count=3,
            missed_pill_reason=[],
            other_missed_pill_reason=None,
        )
        self.data.update(
            subject_visit=self.subject_visit.pk,
            report_datetime=self.subject_visit.report_datetime,
        )

    def test_ok(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        obj = make_recipe("meta_subject.medicationadherence", subject_visit=subject_visit)
        form = MedicationAdherenceForm(instance=obj)
        form.is_valid()
        self.assertIsNone(form._errors)

    def test_pill_count(self):
        """Assert pill count logic.

        Note: tests repeated from edc-adherence
        """
        data = deepcopy(self.data)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        data.update(
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            pill_count_performed=YES,
            pill_count=0,
        )
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)

        data.update(pill_count=1)
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)

        data.update(pill_count_performed=NO, pill_count=0)
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertIn("pill_count", form._errors)

        data.update(pill_count_performed=NO, pill_count=None)
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)



# Generated by Django 4.0.5 on 2022-07-03 08:41

from django.db import migrations
import edc_sites.models
import meta_subject.models.birth_outcomes


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0119_historicalstudymedication_roundup_divisible_by_and_more"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="birthoutcomes",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", meta_subject.models.birth_outcomes.Manager()),
            ],
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="action_identifier",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="action_item_reason",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="parent_action_identifier",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="related_action_identifier",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="subject_visit",
        ),
        migrations.RemoveField(
            model_name="birthoutcomes",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="action_item_reason",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="parent_action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="related_action_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="subject_visit",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="tracking_identifier",
        ),
    ]



from django.apps import AppConfig as DjangoApponfig
from django.db.backends.signals import connection_created
from edc_utils.sqlite import activate_foreign_keys


class AppConfig(DjangoApponfig):
    name = "meta_screening"
    verbose_name = "META Screening"
    screening_age_adult_upper = 99
    screening_age_adult_lower = 18
    include_in_administration_section = True
    has_exportable_data = True
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        connection_created.connect(activate_foreign_keys)



from django import forms
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import InlineCrfModelFormMixin
from edc_form_validators import FormValidatorMixin

from ..models import OtherArvRegimensDetail


class OtherArvRegimensDetailFormValidator(CrfFormValidator):
    def clean(self):
        if not self.cleaned_data.get("DELETE"):
            self.validate_other_specify(
                field="arv_regimen", other_specify_field="other_arv_regimen"
            )
            self.required_if_true(
                self.cleaned_data.get("arv_regimen"),
                field_required="arv_regimen_start_date",
            )


class OtherArvRegimensDetailForm(InlineCrfModelFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = OtherArvRegimensDetailFormValidator

    class Meta:
        model = OtherArvRegimensDetail
        fields = "__all__"



# Generated by Django 3.2.4 on 2021-07-02 02:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0022_auto_20210702_0426"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasted",
            new_name="fasting",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasted_duration_minutes",
            new_name="fasting_duration_minutes",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasted_duration_str",
            new_name="fasting_duration_str",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasted",
            new_name="fasting",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasted_duration_minutes",
            new_name="fasting_duration_minutes",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasted_duration_str",
            new_name="fasting_duration_str",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasted",
            new_name="fasting",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasted_duration_minutes",
            new_name="fasting_duration_minutes",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasted_duration_str",
            new_name="fasting_duration_str",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasted",
            new_name="fasting",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasted_duration_minutes",
            new_name="fasting_duration_minutes",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasted_duration_str",
            new_name="fasting_duration_str",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasted",
            new_name="fasting",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasted_duration_minutes",
            new_name="fasting_duration_minutes",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasted_duration_str",
            new_name="fasting_duration_str",
        ),
    ]



# Generated by Django 3.2.11 on 2022-04-14 14:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0105_auto_20220412_2310"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmedicationadherence",
            name="pill_count",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Number of pills left in the bottle"
            ),
        ),
        migrations.AlterField(
            model_name="medicationadherence",
            name="pill_count",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Number of pills left in the bottle"
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-04-03 11:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0041_auto_20220403_1227"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="fbg2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="fbg2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="fbg2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="fbg2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="fasting_opinion",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="fbg2_performed",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="ogtt2_performed",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="repeat_fasting_opinion",
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="art_six_months",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="meta_phase_two",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Was the subject enrolled in the <u>META Phase 2</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="screening_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the subject given his/her verbal consent to be screened for the <u>META Phase 3</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="staying_nearby_12",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient planning to remain in the catchment area for <u>at least 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 400 copies per ml taken <u>within the last 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="art_six_months",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="meta_phase_two",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Was the subject enrolled in the <u>META Phase 2</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="screening_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the subject given his/her verbal consent to be screened for the <u>META Phase 3</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="staying_nearby_12",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient planning to remain in the catchment area for <u>at least 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 400 copies per ml taken <u>within the last 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="art_six_months",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="meta_phase_two",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Was the subject enrolled in the <u>META Phase 2</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="screening_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the subject given his/her verbal consent to be screened for the <u>META Phase 3</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="staying_nearby_12",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient planning to remain in the catchment area for <u>at least 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 400 copies per ml taken <u>within the last 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="art_six_months",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="meta_phase_two",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Was the subject enrolled in the <u>META Phase 2</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="screening_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the subject given his/her verbal consent to be screened for the <u>META Phase 3</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="staying_nearby_12",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient planning to remain in the catchment area for <u>at least 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 400 copies per ml taken <u>within the last 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="art_six_months",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Has the patient been on anti-retroviral therapy for <u>at least 6 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="meta_phase_two",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Was the subject enrolled in the <u>META Phase 2</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="repeat_glucose_opinion",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)",
                max_length=15,
                verbose_name="In opinion of the clinician, should the glucose measurements be repeated?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="screening_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the subject given his/her verbal consent to be screened for the <u>META Phase 3</u> trial?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="staying_nearby_12",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Is the patient planning to remain in the catchment area for <u>at least 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 400 copies per ml taken <u>within the last 6 months</u>",
            ),
        ),
    ]



from edc_model_admin.dashboard import (
    ModelAdminCrfDashboardMixin,
    ModelAdminSubjectDashboardMixin,
)
from edc_model_admin.history import SimpleHistoryAdmin


class ModelAdminMixin(ModelAdminSubjectDashboardMixin):
    pass


class CrfModelAdminMixin(ModelAdminCrfDashboardMixin):

    pass


class CrfModelAdmin(ModelAdminCrfDashboardMixin, SimpleHistoryAdmin):

    pass



# Generated by Django 4.2 on 2022-08-15 15:33

import django.core.validators
from django.db import migrations, models
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_vitals.models.fields.weight


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0135_auto_20220722_2212"),
    ]

    operations = [
        migrations.AddField(
            model_name="egfrdropnotification",
            name="creatinine_quantifier",
            field=models.CharField(
                blank=True,
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                null=True,
                verbose_name="Quantifier",
            ),
        ),
        migrations.AddField(
            model_name="egfrdropnotification",
            name="egfr_quantifier",
            field=models.CharField(
                blank=True,
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                null=True,
                verbose_name="Quantifier",
            ),
        ),
        migrations.AddField(
            model_name="egfrdropnotification",
            name="egfr_units",
            field=models.CharField(
                blank=True,
                choices=[("mL/min/1.73m2", "mL/min/1.73m2")],
                default="mL/min/1.73m2",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="egfrdropnotification",
            name="egfr_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="eGFR",
            ),
        ),
        migrations.AddField(
            model_name="egfrdropnotification",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalegfrdropnotification",
            name="creatinine_quantifier",
            field=models.CharField(
                blank=True,
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                null=True,
                verbose_name="Quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalegfrdropnotification",
            name="egfr_quantifier",
            field=models.CharField(
                blank=True,
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                null=True,
                verbose_name="Quantifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalegfrdropnotification",
            name="egfr_units",
            field=models.CharField(
                blank=True,
                choices=[("mL/min/1.73m2", "mL/min/1.73m2")],
                default="mL/min/1.73m2",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalegfrdropnotification",
            name="egfr_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="eGFR",
            ),
        ),
        migrations.AddField(
            model_name="historicalegfrdropnotification",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="creatinine_date",
            field=models.DateField(verbose_name="Serum creatinine date"),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        (("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")),
                        (("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")),
                    )
                ],
                default="umol/L",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="report_datetime",
            field=models.DateTimeField(
                default=edc_utils.date.get_utcnow,
                help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report Date",
            ),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="report_status",
            field=models.CharField(
                choices=[
                    ("open", "Open. Some information is still pending."),
                    ("closed", "Closed. This report is complete"),
                ],
                default="New",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="creatinine_date",
            field=models.DateField(verbose_name="Serum creatinine date"),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        (("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")),
                        (("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")),
                    )
                ],
                default="umol/L",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="report_datetime",
            field=models.DateTimeField(
                default=edc_utils.date.get_utcnow,
                help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Report Date",
            ),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="report_status",
            field=models.CharField(
                choices=[
                    ("open", "Open. Some information is still pending."),
                    ("closed", "Closed. This report is complete"),
                ],
                default="New",
                max_length=15,
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-13 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0013_transferreasons_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="abnormalfootappearanceobservations",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="arvregimens",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="baselinesymptoms",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="diabetessymptoms",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="hypertensionmedications",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="nonadherencereasons",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="offstudyreasons",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="oiprophylaxis",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="subjectvisitmissedreasons",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="symptoms",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
        migrations.AddField(
            model_name="transferreasons",
            name="plural_name",
            field=models.CharField(max_length=250, null=True, verbose_name="Plural name"),
        ),
    ]



from tempfile import mkdtemp

from django.test import TestCase
from edc_reportable import ParserError, site_reportables


class TestReportables(TestCase):
    def test(self):
        try:
            from meta_labs import reportables  # noqa
        except ParserError:
            self.fail("ParserError unexpectedly raised.")
        self.assertIsNotNone(site_reportables.get("meta"))
        filename1, filename2 = site_reportables.to_csv("meta", path=mkdtemp())
        print(filename1)
        print(filename2)



import re
from typing import List

from django.db.models import Q
from edc_listboard.views import SubjectListboardView as BaseSubjectListboardView

from meta_edc.meta_version import get_meta_version


class SubjectListboardView(BaseSubjectListboardView):
    def get_context_data(self, **kwargs) -> dict:
        context = super().get_context_data(**kwargs)
        context.update(meta_version=get_meta_version())
        return context

    def extra_search_options(self, search_term) -> List[Q]:
        q_objects = super().extra_search_options(search_term)
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(identity__exact=search_term))
        return q_objects



from dateutil.relativedelta import relativedelta
from edc_facility.constants import FIVE_DAY_CLINIC
from edc_visit_schedule import Schedule, Visit

from ...constants import DELIVERY, SCHEDULE_PREGNANCY
from .crfs import crfs_missed
from .crfs import crfs_prn
from .crfs import crfs_prn as default_crfs_prn
from .crfs_pregnancy import crfs_pregnancy

schedule = Schedule(
    name=SCHEDULE_PREGNANCY,
    verbose_name="Delivery and Birth Outcomes Follow-up",
    onschedule_model="meta_prn.onschedulepregnancy",
    offschedule_model="meta_prn.offschedulepregnancy",
    consent_model="meta_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
    base_timepoint=200,
)


visit = Visit(
    code=DELIVERY,
    title="Delivery",
    timepoint=200,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(months=12),
    crfs=crfs_pregnancy,
    crfs_prn=crfs_prn or default_crfs_prn,
    crfs_missed=crfs_missed,
    facility_name=FIVE_DAY_CLINIC,
)


schedule.add_visit(visit=visit)



# Generated by Django 2.2.7 on 2020-01-23 12:08

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0015_auto_20200120_1943"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultslft",
            name="albumin",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=6,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999),
                ],
                verbose_name="Serum Albumin",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L"), ("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="albumin",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=6,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999),
                ],
                verbose_name="Serum Albumin",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L"), ("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-30 15:19

import edc_visit_tracking.managers
from django.db import migrations

import meta_subject.models.subject_visit


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0046_auto_20200530_1804"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="subjectvisit",
            managers=[
                ("on_site", edc_visit_tracking.managers.VisitCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.VisitModelManager()),
            ],
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-09 23:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0008_auto_20210910_0238"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="protocoldeviationviolation",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Protocol Deviation/Violation",
                "verbose_name_plural": "Protocol Deviations/Violations",
            },
        ),
        migrations.AlterModelManagers(
            name="protocoldeviationviolation",
            managers=[],
        ),
        migrations.AddField(
            model_name="historicalprotocoldeviationviolation",
            name="violation",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_protocol_incident.protocolviolations",
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocoldeviationviolation",
            name="violation_other",
            field=models.CharField(
                blank=True,
                max_length=75,
                null=True,
                verbose_name="If other, please specify",
            ),
        ),
        migrations.AddField(
            model_name="protocoldeviationviolation",
            name="violation",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_protocol_incident.protocolviolations",
            ),
        ),
        migrations.AddField(
            model_name="protocoldeviationviolation",
            name="violation_other",
            field=models.CharField(
                blank=True,
                max_length=75,
                null=True,
                verbose_name="If other, please specify",
            ),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="meta_prn_action_item",
                to="edc_action_item.actionitem",
            ),
        ),
    ]



from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import insulin_panel
from edc_lab_results import BLOOD_RESULTS_INSULIN_ACTION
from edc_lab_results.model_mixins import BloodResultsModelMixin, InsulinModelMixin
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsIns(
    CrfWithActionModelMixin,
    InsulinModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_INSULIN_ACTION
    tracking_identifier_prefix = "IN"
    lab_panel = insulin_panel

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: Insulin"
        verbose_name_plural = "Blood Results: Insulin"



# Generated by Django 3.0.6 on 2020-06-13 17:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0048_auto_20200530_1819"),
    ]

    operations = []



# Generated by Django 3.0.4 on 2020-04-19 20:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0023_auto_20200419_2305"),
    ]

    operations = [
        migrations.AlterField(
            model_name="coronakap",
            name="family_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that <u>someone in your family</u> will get sick from coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="hot_climate",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus does not survive in the hot climate",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="know_other_symptoms",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Do you know of any other symptoms of coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="lives_on_materials",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus can live on clothes, plastics, cardboard for a day or more",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="personal_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that <u>you</u> will get sick from coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="severity_age",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in older people than children",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="severity_diabetes_hypertension",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in people with <u>diabetes</u> and/or <u>hypertension</u>",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="severity_hiv",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in people with <u>HIV infection</u>",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="spread_asymptomatic",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="People can transmit coronavirus even when they do not appear to be sick",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="spread_droplets",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus spreads by droplets from cough and sneezes from people infected with coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="spread_sick",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="People can transmit coronavirus when they are sick ",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="spread_touch",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus can spread by people touching each other",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="spread_touch2",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="You can catch coronavirus if you touch an infected area and then touch your face or eyes",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="stop_chronic_meds",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                help_text="For example, medicines for diabetes, hypertension and/or HIV",
                max_length=25,
                verbose_name="Stop taking your chronic disease medicines",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="symptoms_other",
            field=models.TextField(
                blank=True,
                max_length=250,
                null=True,
                verbose_name="Please list any other symptoms of coronavirus that you are aware of:",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="family_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that <u>someone in your family</u> will get sick from coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="hot_climate",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus does not survive in the hot climate",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="know_other_symptoms",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Do you know of any other symptoms of coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="lives_on_materials",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus can live on clothes, plastics, cardboard for a day or more",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="personal_infection_likelihood",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                max_length=25,
                verbose_name="How likely do you think it is that <u>you</u> will get sick from coronavirus?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="severity_age",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in older people than children",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="severity_diabetes_hypertension",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in people with <u>diabetes</u> and/or <u>hypertension</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="severity_hiv",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus is more severe in people with <u>HIV infection</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="spread_asymptomatic",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="People can transmit coronavirus even when they do not appear to be sick",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="spread_droplets",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus spreads by droplets from cough and sneezes from people infected with coronavirus",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="spread_sick",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="People can transmit coronavirus when they are sick ",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="spread_touch",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="Coronavirus can spread by people touching each other",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="spread_touch2",
            field=models.CharField(
                choices=[
                    ("true", "True"),
                    ("false", "False"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="You can catch coronavirus if you touch an infected area and then touch your face or eyes",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="stop_chronic_meds",
            field=models.CharField(
                choices=[
                    ("very", "Very likely"),
                    ("somewhat", "Somewhat likely"),
                    ("unlikely", "Not very likely, unlikely"),
                    ("not_at_all", "Not at all"),
                ],
                help_text="For example, medicines for diabetes, hypertension and/or HIV",
                max_length=25,
                verbose_name="Stop taking your chronic disease medicines",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="symptoms_other",
            field=models.TextField(
                blank=True,
                max_length=250,
                null=True,
                verbose_name="Please list any other symptoms of coronavirus that you are aware of:",
            ),
        ),
    ]



# Generated by Django 2.2.7 on 2020-01-20 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0018_auto_20200118_1854"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="unsuitable_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="unsuitable_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="unsuitable_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="unsuitable_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="unsuitable_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="reasons_unsuitable",
            field=models.TextField(
                blank=True,
                max_length=150,
                null=True,
                verbose_name="Reason not suitable for the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="reasons_unsuitable",
            field=models.TextField(
                blank=True,
                max_length=150,
                null=True,
                verbose_name="Reason not suitable for the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="reasons_unsuitable",
            field=models.TextField(
                blank=True,
                max_length=150,
                null=True,
                verbose_name="Reason not suitable for the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="reasons_unsuitable",
            field=models.TextField(
                blank=True,
                max_length=150,
                null=True,
                verbose_name="Reason not suitable for the study",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="reasons_unsuitable",
            field=models.TextField(
                blank=True,
                max_length=150,
                null=True,
                verbose_name="Reason not suitable for the study",
            ),
        ),
    ]



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import DeathReportTmgModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_ae_admin
from ..forms import DeathReportTmgForm
from ..models import DeathReportTmg


@admin.register(DeathReportTmg, site=meta_ae_admin)
class DeathReportTmgAdmin(DeathReportTmgModelAdminMixin, SimpleHistoryAdmin):

    form = DeathReportTmgForm



# Generated by Django 3.2.13 on 2022-09-07 12:05
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_appointment.utils import get_next_appointment


def update_refill_end_datetimes(apps, schema_editor):
    rx_model_cls = apps.get_model("edc_pharmacy.rx")
    rx_refill_model_cls = apps.get_model("edc_pharmacy.rxrefill")
    appointment_model_cls = apps.get_model("edc_appointment.appointment")
    subject_visit_model_cls = apps.get_model("meta_subject.subjectvisit")
    model_cls = apps.get_model("meta_subject.studymedication")
    for obj in model_cls._default_manager.filter(refill_end_datetime__isnull=True).order_by(
        "refill_start_datetime"
    ):
        subject_visit = subject_visit_model_cls._default_manager.get(id=obj.subject_visit_id)
        appointment = appointment_model_cls._default_manager.get(
            id=subject_visit.appointment_id
        )
        try:
            rx_refill = rx_refill_model_cls._default_manager.get(
                refill_start_datetime=obj.refill_start_datetime,
                rx__subject_identifier=appointment.subject_identifier,
            )
        except ObjectDoesNotExist:
            rx_refill = None
        try:
            obj.refill_end_datetime = get_next_appointment(
                appointment, include_interim=True
            ).appt_datetime
        except AttributeError:
            print(f"Unable to update. No next appt. Got {obj}.")
        else:
            obj.save_base(update_fields=["refill_end_datetime"])
            if rx_refill:
                rx_refill.refill_end_datetime = obj.refill_end_datetime
                rx_refill.save_base(update_fields=["refill_end_datetime"])


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0146_auto_20220907_0207"),
    ]

    operations = [migrations.RunPython(update_refill_end_datetimes)]
